-- $Source: /user/cp_source/ObjectAda/rts/common/strings_characters/ada.strings.maps.constants.dat,v $ 
-- $Revision: 1.2 $ $Date: 1999/06/14 23:44:19 $ $Author: michael $ 
--        Copyright (C) 1995 by Intermetrics, Inc.

with Ada.Characters.Latin_1;

package Ada.Strings.Maps.Constants is
    pragma Suppress( Elaboration_Check );
    pragma Preelaborate (Constants);

    Control_Set           : constant Character_Set;
    Graphic_Set           : constant Character_Set;
    Letter_Set            : constant Character_Set;
    Lower_Set             : constant Character_Set;
    Upper_Set             : constant Character_Set;
    Basic_Set             : constant Character_Set;
    Decimal_Digit_Set     : constant Character_Set;
    Hexadecimal_Digit_Set : constant Character_Set;
    Alphanumeric_Set      : constant Character_Set;
    Special_Set           : constant Character_Set;
    ISO_646_Set           : constant Character_Set;

    Lower_Case_Map        : constant Character_Mapping;
    --  Maps to lower case for letters, else identity

    Upper_Case_Map        : constant Character_Mapping;
    --  Maps to upper case for letters, else identity

    Basic_Map             : constant Character_Mapping;
    --  Maps to basic letters for letters, else identity

private

    use Ada.Characters.Latin_1;

    Control_Set           : constant Character_Set := (
        NUL                 .. US                  => True,
        DEL                 .. APC                 => True,
        others => False);

    Graphic_Set           : constant Character_Set := (
        ' '                 .. '~'                 => True,
        No_Break_Space      .. LC_Y_Diaeresis      => True,
        others => False);

    Letter_Set            : constant Character_Set := (
        'A'                 .. 'Z'                 => True,
        'a'                 .. 'z'                 => True,
        UC_A_Grave          .. UC_O_Diaeresis      => True,
        UC_O_Oblique_Stroke .. LC_O_Diaeresis      => True,
        LC_O_Oblique_Stroke .. LC_Y_Diaeresis      => True,
        others => False);

    Lower_Set             : constant Character_Set := (
        'a'                 .. 'z'                 => True,
        LC_German_Sharp_S   .. LC_O_Diaeresis      => True,
        LC_O_Oblique_Stroke .. LC_Y_Diaeresis      => True,
        others => False);

    Upper_Set             : constant Character_Set := (
        'A'                 .. 'Z'                 => True,
        UC_A_Grave          .. UC_O_Diaeresis      => True,
        UC_O_Oblique_Stroke .. UC_Icelandic_Thorn  => True,
        others => False);

    Basic_Set             : constant Character_Set := (
        'A'                 .. 'Z'                 => True,
        'a'                 .. 'z'                 => True,
        UC_AE_Diphthong     .. UC_AE_Diphthong     => True,
        LC_AE_Diphthong     .. LC_AE_Diphthong     => True,
        LC_German_Sharp_S   .. LC_German_Sharp_S   => True,
        UC_Icelandic_Thorn  .. UC_Icelandic_Thorn  => True,
        LC_Icelandic_Thorn  .. LC_Icelandic_Thorn  => True,
        UC_Icelandic_Eth    .. UC_Icelandic_Eth    => True,
        LC_Icelandic_Eth    .. LC_Icelandic_Eth    => True,
        others => False);

    Decimal_Digit_Set     : constant Character_Set := (
        '0'                 .. '9'                 => True,
        others => False);

    Hexadecimal_Digit_Set : constant Character_Set := (
        '0'                 .. '9'                 => True,
        'A'                 .. 'F'                 => True,
        'a'                 .. 'f'                 => True,
        others => False);

    Alphanumeric_Set      : constant Character_Set := (
        '0'                 .. '9'                 => True,
        'A'                 .. 'Z'                 => True,
        'a'                 .. 'z'                 => True,
        UC_A_Grave          .. UC_O_Diaeresis      => True,
        UC_O_Oblique_Stroke .. LC_O_Diaeresis      => True,
        LC_O_Oblique_Stroke .. LC_Y_Diaeresis      => True,
        others => False);

    Special_Set           : constant Character_Set := (
        ' '                 .. '/'                 => True,
        ':'                 .. '@'                 => True,
        '['                 .. '`'                 => True,
        '{'                 .. '~'                 => True,
        No_Break_Space      .. Inverted_Question   => True,
        Multiplication_Sign .. Multiplication_Sign => True,
        Division_Sign       .. Division_Sign       => True,
        others => False);

    ISO_646_Set           : constant Character_Set := (
        NUL                 .. DEL                 => True,
        others => False);

    Lower_Case_Map : constant Character_Mapping := (
        -- Control characters:
  
        NUL                         ,  --             0
        SOH                         ,  --             1
        STX                         ,  --             2
        ETX                         ,  --             3
        EOT                         ,  --             4
        ENQ                         ,  --             5
        ACK                         ,  --             6
        BEL                         ,  --             7
        BS                          ,  --             8
        HT                          ,  --             9
        LF                          ,  --            10
        VT                          ,  --            11
        FF                          ,  --            12
        CR                          ,  --            13
        SO                          ,  --            14
        SI                          ,  --            15
  
        DLE                         ,  --            16
        DC1                         ,  --            17
        DC2                         ,  --            18
        DC3                         ,  --            19
        DC4                         ,  --            20
        NAK                         ,  --            21
        SYN                         ,  --            22
        ETB                         ,  --            23
        CAN                         ,  --            24
        EM                          ,  --            25
        SUB                         ,  --            26
        ESC                         ,  --            27
        FS                          ,  --            28
        GS                          ,  --            29
        RS                          ,  --            30
        US                          ,  --            31
  
        -- ISO 646 graphic characters:
  
        Space                       ,  -- ' '        32
        Exclamation                 ,  -- '!'        33
        Quotation                   ,  -- '"'        34
        Number_Sign                 ,  -- '#'        35
        Dollar_Sign                 ,  -- '$'        36
        Percent_Sign                ,  -- '%'        37
        Ampersand                   ,  -- '&'        38
        Apostrophe                  ,  -- '''        39
        Left_Parenthesis            ,  -- '('        40
        Right_Parenthesis           ,  -- ')'        41
        Asterisk                    ,  -- '*'        42
        Plus_Sign                   ,  -- '+'        43
        Comma                       ,  -- ','        44
        Hyphen                      ,  -- '-'        45
        Full_Stop                   ,  -- '.'        46
        Solidus                     ,  -- '/'        47
  
        '0'                         ,  --            48
        '1'                         ,  --            49
        '2'                         ,  --            50
        '3'                         ,  --            51
        '4'                         ,  --            52
        '5'                         ,  --            53
        '6'                         ,  --            54
        '7'                         ,  --            55
        '8'                         ,  --            56
        '9'                         ,  --            57
        Colon                       ,  -- ':'        58
        Semicolon                   ,  -- ';'        59
        Less_Than_Sign              ,  -- '<'        60
        Equals_Sign                 ,  -- '='        61
        Greater_Than_Sign           ,  -- '>'        62
        Question                    ,  -- '?'        63
  
        Commercial_At               ,  -- '@'        64
        LC_A                        ,  -- 'a'        65
        LC_B                        ,  -- 'b'        66
        LC_C                        ,  -- 'c'        67
        LC_D                        ,  -- 'd'        68
        LC_E                        ,  -- 'e'        69
        LC_F                        ,  -- 'f'        70
        LC_G                        ,  -- 'g'        71
        LC_H                        ,  -- 'h'        72
        LC_I                        ,  -- 'i'        73
        LC_J                        ,  -- 'j'        74
        LC_K                        ,  -- 'k'        75
        LC_L                        ,  -- 'l'        76
        LC_M                        ,  -- 'm'        77
        LC_N                        ,  -- 'n'        78
        LC_O                        ,  -- 'o'        79
  
        LC_P                        ,  -- 'p'        80
        LC_Q                        ,  -- 'q'        81
        LC_R                        ,  -- 'r'        82
        LC_S                        ,  -- 's'        83
        LC_T                        ,  -- 't'        84
        LC_U                        ,  -- 'u'        85
        LC_V                        ,  -- 'v'        86
        LC_W                        ,  -- 'w'        87
        LC_X                        ,  -- 'x'        88
        LC_Y                        ,  -- 'y'        89
        LC_Z                        ,  -- 'z'        90
        Left_Square_Bracket         ,  -- '['        91
        Reverse_Solidus             ,  -- '\'        92
        Right_Square_Bracket        ,  -- ']'        93
        Circumflex                  ,  -- '^'        94
        Low_Line                    ,  -- '_'        95
  
        Grave                       ,  -- '`'        96
        LC_A                        ,  -- 'a'        97
        LC_B                        ,  -- 'b'        98
        LC_C                        ,  -- 'c'        99
        LC_D                        ,  -- 'd'       100
        LC_E                        ,  -- 'e'       101
        LC_F                        ,  -- 'f'       102
        LC_G                        ,  -- 'g'       103
        LC_H                        ,  -- 'h'       104
        LC_I                        ,  -- 'i'       105
        LC_J                        ,  -- 'j'       106
        LC_K                        ,  -- 'k'       107
        LC_L                        ,  -- 'l'       108
        LC_M                        ,  -- 'm'       109
        LC_N                        ,  -- 'n'       110
        LC_O                        ,  -- 'o'       111
  
        LC_P                        ,  -- 'p'       112
        LC_Q                        ,  -- 'q'       113
        LC_R                        ,  -- 'r'       114
        LC_S                        ,  -- 's'       115
        LC_T                        ,  -- 't'       116
        LC_U                        ,  -- 'u'       117
        LC_V                        ,  -- 'v'       118
        LC_W                        ,  -- 'w'       119
        LC_X                        ,  -- 'x'       120
        LC_Y                        ,  -- 'y'       121
        LC_Z                        ,  -- 'z'       122
        Left_Curly_Bracket          ,  -- '{'       123
        Vertical_Line               ,  -- '|'       124
        Right_Curly_Bracket         ,  -- '}'       125
        Tilde                       ,  -- '~'       126
        DEL                         ,  --           127
  
        -- ISO 6429 control characters:
  
        Reserved_128                ,  --           128
        Reserved_129                ,  --           129
        BPH                         ,  --           130
        NBH                         ,  --           131
        Reserved_132                ,  --           132
        NEL                         ,  --           133
        SSA                         ,  --           134
        ESA                         ,  --           135
        HTS                         ,  --           136
        HTJ                         ,  --           137
        VTS                         ,  --           138
        PLD                         ,  --           139
        PLU                         ,  --           140
        RI                          ,  --           141
        SS2                         ,  --           142
        SS3                         ,  --           143
  
        DCS                         ,  --           144
        PU1                         ,  --           145
        PU2                         ,  --           146
        STS                         ,  --           147
        CCH                         ,  --           148
        MW                          ,  --           149
        SPA                         ,  --           150
        EPA                         ,  --           151
        SOS                         ,  --           152
        Reserved_153                ,  --           153
        SCI                         ,  --           154
        CSI                         ,  --           155
        ST                          ,  --           156
        OSC                         ,  --           157
        PM                          ,  --           158
        APC                         ,  --           159
  
        -- Other graphic characters:
  
        No_Break_Space              ,  --           160
        Inverted_Exclamation        ,  --           161
        Cent_Sign                   ,  --           162
        Pound_Sign                  ,  --           163
        Currency_Sign               ,  --           164
        Yen_Sign                    ,  --           165
        Broken_Bar                  ,  --           166
        Section_Sign                ,  --           167
        Diaeresis                   ,  --           168
        Copyright_Sign              ,  --           169
        Feminine_Ordinal_Indicator  ,  --           170
        Left_Angle_Quotation        ,  --           171
        Not_Sign                    ,  --           172
        Soft_Hyphen                 ,  --           173
        Registered_Trade_Mark_Sign  ,  --           174
        Macron                      ,  --           175
  
        Degree_Sign                 ,  --           176
        Plus_Minus_Sign             ,  --           177
        Superscript_Two             ,  --           178
        Superscript_Three           ,  --           179
        Acute                       ,  --           180
        Micro_Sign                  ,  --           181
        Pilcrow_Sign                ,  --           182
        Middle_Dot                  ,  --           183
        Cedilla                     ,  --           184
        Superscript_One             ,  --           185
        Masculine_Ordinal_Indicator ,  --           186
        Right_Angle_Quotation       ,  --           187
        Fraction_One_Quarter        ,  --           188
        Fraction_One_Half           ,  --           189
        Fraction_Three_Quarters     ,  --           190
        Inverted_Question           ,  --           191
  
        LC_A_Grave                  ,  --           192
        LC_A_Acute                  ,  --           193
        LC_A_Circumflex             ,  --           194
        LC_A_Tilde                  ,  --           195
        LC_A_Diaeresis              ,  --           196
        LC_A_Ring                   ,  --           197
        LC_AE_Diphthong             ,  --           198
        LC_C_Cedilla                ,  --           199
        LC_E_Grave                  ,  --           200
        LC_E_Acute                  ,  --           201
        LC_E_Circumflex             ,  --           202
        LC_E_Diaeresis              ,  --           203
        LC_I_Grave                  ,  --           204
        LC_I_Acute                  ,  --           205
        LC_I_Circumflex             ,  --           206
        LC_I_Diaeresis              ,  --           207
  
        LC_Icelandic_Eth            ,  --           208
        LC_N_Tilde                  ,  --           209
        LC_O_Grave                  ,  --           210
        LC_O_Acute                  ,  --           211
        LC_O_Circumflex             ,  --           212
        LC_O_Tilde                  ,  --           213
        LC_O_Diaeresis              ,  --           214
        Multiplication_Sign         ,  --           215
        LC_O_Oblique_Stroke         ,  --           216
        LC_U_Grave                  ,  --           217
        LC_U_Acute                  ,  --           218
        LC_U_Circumflex             ,  --           219
        LC_U_Diaeresis              ,  --           220
        LC_Y_Acute                  ,  --           221
        LC_Icelandic_Thorn          ,  --           222
        LC_German_Sharp_S           ,  --           223
  
        LC_A_Grave                  ,  --           224
        LC_A_Acute                  ,  --           225
        LC_A_Circumflex             ,  --           226
        LC_A_Tilde                  ,  --           227
        LC_A_Diaeresis              ,  --           228
        LC_A_Ring                   ,  --           229
        LC_AE_Diphthong             ,  --           230
        LC_C_Cedilla                ,  --           231
        LC_E_Grave                  ,  --           232
        LC_E_Acute                  ,  --           233
        LC_E_Circumflex             ,  --           234
        LC_E_Diaeresis              ,  --           235
        LC_I_Grave                  ,  --           236
        LC_I_Acute                  ,  --           237
        LC_I_Circumflex             ,  --           238
        LC_I_Diaeresis              ,  --           239
  
        LC_Icelandic_Eth            ,  --           240
        LC_N_Tilde                  ,  --           241
        LC_O_Grave                  ,  --           242
        LC_O_Acute                  ,  --           243
        LC_O_Circumflex             ,  --           244
        LC_O_Tilde                  ,  --           245
        LC_O_Diaeresis              ,  --           246
        Division_Sign               ,  --           247
        LC_O_Oblique_Stroke         ,  --           248
        LC_U_Grave                  ,  --           249
        LC_U_Acute                  ,  --           250
        LC_U_Circumflex             ,  --           251
        LC_U_Diaeresis              ,  --           252
        LC_Y_Acute                  ,  --           253
        LC_Icelandic_Thorn          ,  --           254
        LC_Y_Diaeresis              ); --           255

    Upper_Case_Map : constant Character_Mapping := (
        -- Control characters:
  
        NUL                         ,  --             0
        SOH                         ,  --             1
        STX                         ,  --             2
        ETX                         ,  --             3
        EOT                         ,  --             4
        ENQ                         ,  --             5
        ACK                         ,  --             6
        BEL                         ,  --             7
        BS                          ,  --             8
        HT                          ,  --             9
        LF                          ,  --            10
        VT                          ,  --            11
        FF                          ,  --            12
        CR                          ,  --            13
        SO                          ,  --            14
        SI                          ,  --            15
  
        DLE                         ,  --            16
        DC1                         ,  --            17
        DC2                         ,  --            18
        DC3                         ,  --            19
        DC4                         ,  --            20
        NAK                         ,  --            21
        SYN                         ,  --            22
        ETB                         ,  --            23
        CAN                         ,  --            24
        EM                          ,  --            25
        SUB                         ,  --            26
        ESC                         ,  --            27
        FS                          ,  --            28
        GS                          ,  --            29
        RS                          ,  --            30
        US                          ,  --            31
  
        -- ISO 646 graphic characters:
  
        Space                       ,  -- ' '        32
        Exclamation                 ,  -- '!'        33
        Quotation                   ,  -- '"'        34
        Number_Sign                 ,  -- '#'        35
        Dollar_Sign                 ,  -- '$'        36
        Percent_Sign                ,  -- '%'        37
        Ampersand                   ,  -- '&'        38
        Apostrophe                  ,  -- '''        39
        Left_Parenthesis            ,  -- '('        40
        Right_Parenthesis           ,  -- ')'        41
        Asterisk                    ,  -- '*'        42
        Plus_Sign                   ,  -- '+'        43
        Comma                       ,  -- ','        44
        Hyphen                      ,  -- '-'        45
        Full_Stop                   ,  -- '.'        46
        Solidus                     ,  -- '/'        47
  
        '0'                         ,  --            48
        '1'                         ,  --            49
        '2'                         ,  --            50
        '3'                         ,  --            51
        '4'                         ,  --            52
        '5'                         ,  --            53
        '6'                         ,  --            54
        '7'                         ,  --            55
        '8'                         ,  --            56
        '9'                         ,  --            57
        Colon                       ,  -- ':'        58
        Semicolon                   ,  -- ';'        59
        Less_Than_Sign              ,  -- '<'        60
        Equals_Sign                 ,  -- '='        61
        Greater_Than_Sign           ,  -- '>'        62
        Question                    ,  -- '?'        63
  
        Commercial_At               ,  -- '@'        64
        'A'                         ,  --            65
        'B'                         ,  --            66
        'C'                         ,  --            67
        'D'                         ,  --            68
        'E'                         ,  --            69
        'F'                         ,  --            70
        'G'                         ,  --            71
        'H'                         ,  --            72
        'I'                         ,  --            73
        'J'                         ,  --            74
        'K'                         ,  --            75
        'L'                         ,  --            76
        'M'                         ,  --            77
        'N'                         ,  --            78
        'O'                         ,  --            79
  
        'P'                         ,  --            80
        'Q'                         ,  --            81
        'R'                         ,  --            82
        'S'                         ,  --            83
        'T'                         ,  --            84
        'U'                         ,  --            85
        'V'                         ,  --            86
        'W'                         ,  --            87
        'X'                         ,  --            88
        'Y'                         ,  --            89
        'Z'                         ,  --            90
        Left_Square_Bracket         ,  -- '['        91
        Reverse_Solidus             ,  -- '\'        92
        Right_Square_Bracket        ,  -- ']'        93
        Circumflex                  ,  -- '^'        94
        Low_Line                    ,  -- '_'        95
  
        Grave                       ,  -- '`'        96
        'A'                         ,  --            97
        'B'                         ,  --            98
        'C'                         ,  --            99
        'D'                         ,  --           100
        'E'                         ,  --           101
        'F'                         ,  --           102
        'G'                         ,  --           103
        'H'                         ,  --           104
        'I'                         ,  --           105
        'J'                         ,  --           106
        'K'                         ,  --           107
        'L'                         ,  --           108
        'M'                         ,  --           109
        'N'                         ,  --           110
        'O'                         ,  --           111
  
        'P'                         ,  --           112
        'Q'                         ,  --           113
        'R'                         ,  --           114
        'S'                         ,  --           115
        'T'                         ,  --           116
        'U'                         ,  --           117
        'V'                         ,  --           118
        'W'                         ,  --           119
        'X'                         ,  --           120
        'Y'                         ,  --           121
        'Z'                         ,  --           122
        Left_Curly_Bracket          ,  -- '{'       123
        Vertical_Line               ,  -- '|'       124
        Right_Curly_Bracket         ,  -- '}'       125
        Tilde                       ,  -- '~'       126
        DEL                         ,  --           127
  
        -- ISO 6429 control characters:
  
        Reserved_128                ,  --           128
        Reserved_129                ,  --           129
        BPH                         ,  --           130
        NBH                         ,  --           131
        Reserved_132                ,  --           132
        NEL                         ,  --           133
        SSA                         ,  --           134
        ESA                         ,  --           135
        HTS                         ,  --           136
        HTJ                         ,  --           137
        VTS                         ,  --           138
        PLD                         ,  --           139
        PLU                         ,  --           140
        RI                          ,  --           141
        SS2                         ,  --           142
        SS3                         ,  --           143
  
        DCS                         ,  --           144
        PU1                         ,  --           145
        PU2                         ,  --           146
        STS                         ,  --           147
        CCH                         ,  --           148
        MW                          ,  --           149
        SPA                         ,  --           150
        EPA                         ,  --           151
        SOS                         ,  --           152
        Reserved_153                ,  --           153
        SCI                         ,  --           154
        CSI                         ,  --           155
        ST                          ,  --           156
        OSC                         ,  --           157
        PM                          ,  --           158
        APC                         ,  --           159
  
        -- Other graphic characters:
  
        No_Break_Space              ,  --           160
        Inverted_Exclamation        ,  --           161
        Cent_Sign                   ,  --           162
        Pound_Sign                  ,  --           163
        Currency_Sign               ,  --           164
        Yen_Sign                    ,  --           165
        Broken_Bar                  ,  --           166
        Section_Sign                ,  --           167
        Diaeresis                   ,  --           168
        Copyright_Sign              ,  --           169
        Feminine_Ordinal_Indicator  ,  --           170
        Left_Angle_Quotation        ,  --           171
        Not_Sign                    ,  --           172
        Soft_Hyphen                 ,  --           173
        Registered_Trade_Mark_Sign  ,  --           174
        Macron                      ,  --           175
  
        Degree_Sign                 ,  --           176
        Plus_Minus_Sign             ,  --           177
        Superscript_Two             ,  --           178
        Superscript_Three           ,  --           179
        Acute                       ,  --           180
        Micro_Sign                  ,  --           181
        Pilcrow_Sign                ,  --           182
        Middle_Dot                  ,  --           183
        Cedilla                     ,  --           184
        Superscript_One             ,  --           185
        Masculine_Ordinal_Indicator ,  --           186
        Right_Angle_Quotation       ,  --           187
        Fraction_One_Quarter        ,  --           188
        Fraction_One_Half           ,  --           189
        Fraction_Three_Quarters     ,  --           190
        Inverted_Question           ,  --           191
  
        UC_A_Grave                  ,  --           192
        UC_A_Acute                  ,  --           193
        UC_A_Circumflex             ,  --           194
        UC_A_Tilde                  ,  --           195
        UC_A_Diaeresis              ,  --           196
        UC_A_Ring                   ,  --           197
        UC_AE_Diphthong             ,  --           198
        UC_C_Cedilla                ,  --           199
        UC_E_Grave                  ,  --           200
        UC_E_Acute                  ,  --           201
        UC_E_Circumflex             ,  --           202
        UC_E_Diaeresis              ,  --           203
        UC_I_Grave                  ,  --           204
        UC_I_Acute                  ,  --           205
        UC_I_Circumflex             ,  --           206
        UC_I_Diaeresis              ,  --           207
  
        UC_Icelandic_Eth            ,  --           208
        UC_N_Tilde                  ,  --           209
        UC_O_Grave                  ,  --           210
        UC_O_Acute                  ,  --           211
        UC_O_Circumflex             ,  --           212
        UC_O_Tilde                  ,  --           213
        UC_O_Diaeresis              ,  --           214
        Multiplication_Sign         ,  --           215
        UC_O_Oblique_Stroke         ,  --           216
        UC_U_Grave                  ,  --           217
        UC_U_Acute                  ,  --           218
        UC_U_Circumflex             ,  --           219
        UC_U_Diaeresis              ,  --           220
        UC_Y_Acute                  ,  --           221
        UC_Icelandic_Thorn          ,  --           222
        LC_German_Sharp_S           ,  --           223
  
        UC_A_Grave                  ,  --           224
        UC_A_Acute                  ,  --           225
        UC_A_Circumflex             ,  --           226
        UC_A_Tilde                  ,  --           227
        UC_A_Diaeresis              ,  --           228
        UC_A_Ring                   ,  --           229
        UC_AE_Diphthong             ,  --           230
        UC_C_Cedilla                ,  --           231
        UC_E_Grave                  ,  --           232
        UC_E_Acute                  ,  --           233
        UC_E_Circumflex             ,  --           234
        UC_E_Diaeresis              ,  --           235
        UC_I_Grave                  ,  --           236
        UC_I_Acute                  ,  --           237
        UC_I_Circumflex             ,  --           238
        UC_I_Diaeresis              ,  --           239
  
        UC_Icelandic_Eth            ,  --           240
        UC_N_Tilde                  ,  --           241
        UC_O_Grave                  ,  --           242
        UC_O_Acute                  ,  --           243
        UC_O_Circumflex             ,  --           244
        UC_O_Tilde                  ,  --           245
        UC_O_Diaeresis              ,  --           246
        Division_Sign               ,  --           247
        UC_O_Oblique_Stroke         ,  --           248
        UC_U_Grave                  ,  --           249
        UC_U_Acute                  ,  --           250
        UC_U_Circumflex             ,  --           251
        UC_U_Diaeresis              ,  --           252
        UC_Y_Acute                  ,  --           253
        UC_Icelandic_Thorn          ,  --           254
        LC_Y_Diaeresis              ); --           255

    Basic_Map : constant Character_Mapping := (
        -- Control characters:
  
        NUL                         ,  --             0
        SOH                         ,  --             1
        STX                         ,  --             2
        ETX                         ,  --             3
        EOT                         ,  --             4
        ENQ                         ,  --             5
        ACK                         ,  --             6
        BEL                         ,  --             7
        BS                          ,  --             8
        HT                          ,  --             9
        LF                          ,  --            10
        VT                          ,  --            11
        FF                          ,  --            12
        CR                          ,  --            13
        SO                          ,  --            14
        SI                          ,  --            15
  
        DLE                         ,  --            16
        DC1                         ,  --            17
        DC2                         ,  --            18
        DC3                         ,  --            19
        DC4                         ,  --            20
        NAK                         ,  --            21
        SYN                         ,  --            22
        ETB                         ,  --            23
        CAN                         ,  --            24
        EM                          ,  --            25
        SUB                         ,  --            26
        ESC                         ,  --            27
        FS                          ,  --            28
        GS                          ,  --            29
        RS                          ,  --            30
        US                          ,  --            31
  
        -- ISO 646 graphic characters:
  
        Space                       ,  -- ' '        32
        Exclamation                 ,  -- '!'        33
        Quotation                   ,  -- '"'        34
        Number_Sign                 ,  -- '#'        35
        Dollar_Sign                 ,  -- '$'        36
        Percent_Sign                ,  -- '%'        37
        Ampersand                   ,  -- '&'        38
        Apostrophe                  ,  -- '''        39
        Left_Parenthesis            ,  -- '('        40
        Right_Parenthesis           ,  -- ')'        41
        Asterisk                    ,  -- '*'        42
        Plus_Sign                   ,  -- '+'        43
        Comma                       ,  -- ','        44
        Hyphen                      ,  -- '-'        45
        Full_Stop                   ,  -- '.'        46
        Solidus                     ,  -- '/'        47
  
        '0'                         ,  --            48
        '1'                         ,  --            49
        '2'                         ,  --            50
        '3'                         ,  --            51
        '4'                         ,  --            52
        '5'                         ,  --            53
        '6'                         ,  --            54
        '7'                         ,  --            55
        '8'                         ,  --            56
        '9'                         ,  --            57
        Colon                       ,  -- ':'        58
        Semicolon                   ,  -- ';'        59
        Less_Than_Sign              ,  -- '<'        60
        Equals_Sign                 ,  -- '='        61
        Greater_Than_Sign           ,  -- '>'        62
        Question                    ,  -- '?'        63
  
        Commercial_At               ,  -- '@'        64
        'A'                         ,  --            65
        'B'                         ,  --            66
        'C'                         ,  --            67
        'D'                         ,  --            68
        'E'                         ,  --            69
        'F'                         ,  --            70
        'G'                         ,  --            71
        'H'                         ,  --            72
        'I'                         ,  --            73
        'J'                         ,  --            74
        'K'                         ,  --            75
        'L'                         ,  --            76
        'M'                         ,  --            77
        'N'                         ,  --            78
        'O'                         ,  --            79
  
        'P'                         ,  --            80
        'Q'                         ,  --            81
        'R'                         ,  --            82
        'S'                         ,  --            83
        'T'                         ,  --            84
        'U'                         ,  --            85
        'V'                         ,  --            86
        'W'                         ,  --            87
        'X'                         ,  --            88
        'Y'                         ,  --            89
        'Z'                         ,  --            90
        Left_Square_Bracket         ,  -- '['        91
        Reverse_Solidus             ,  -- '\'        92
        Right_Square_Bracket        ,  -- ']'        93
        Circumflex                  ,  -- '^'        94
        Low_Line                    ,  -- '_'        95
  
        Grave                       ,  -- '`'        96
        LC_A                        ,  -- 'a'        97
        LC_B                        ,  -- 'b'        98
        LC_C                        ,  -- 'c'        99
        LC_D                        ,  -- 'd'       100
        LC_E                        ,  -- 'e'       101
        LC_F                        ,  -- 'f'       102
        LC_G                        ,  -- 'g'       103
        LC_H                        ,  -- 'h'       104
        LC_I                        ,  -- 'i'       105
        LC_J                        ,  -- 'j'       106
        LC_K                        ,  -- 'k'       107
        LC_L                        ,  -- 'l'       108
        LC_M                        ,  -- 'm'       109
        LC_N                        ,  -- 'n'       110
        LC_O                        ,  -- 'o'       111
  
        LC_P                        ,  -- 'p'       112
        LC_Q                        ,  -- 'q'       113
        LC_R                        ,  -- 'r'       114
        LC_S                        ,  -- 's'       115
        LC_T                        ,  -- 't'       116
        LC_U                        ,  -- 'u'       117
        LC_V                        ,  -- 'v'       118
        LC_W                        ,  -- 'w'       119
        LC_X                        ,  -- 'x'       120
        LC_Y                        ,  -- 'y'       121
        LC_Z                        ,  -- 'z'       122
        Left_Curly_Bracket          ,  -- '{'       123
        Vertical_Line               ,  -- '|'       124
        Right_Curly_Bracket         ,  -- '}'       125
        Tilde                       ,  -- '~'       126
        DEL                         ,  --           127
  
        -- ISO 6429 control characters:
  
        Reserved_128                ,  --           128
        Reserved_129                ,  --           129
        BPH                         ,  --           130
        NBH                         ,  --           131
        Reserved_132                ,  --           132
        NEL                         ,  --           133
        SSA                         ,  --           134
        ESA                         ,  --           135
        HTS                         ,  --           136
        HTJ                         ,  --           137
        VTS                         ,  --           138
        PLD                         ,  --           139
        PLU                         ,  --           140
        RI                          ,  --           141
        SS2                         ,  --           142
        SS3                         ,  --           143
  
        DCS                         ,  --           144
        PU1                         ,  --           145
        PU2                         ,  --           146
        STS                         ,  --           147
        CCH                         ,  --           148
        MW                          ,  --           149
        SPA                         ,  --           150
        EPA                         ,  --           151
        SOS                         ,  --           152
        Reserved_153                ,  --           153
        SCI                         ,  --           154
        CSI                         ,  --           155
        ST                          ,  --           156
        OSC                         ,  --           157
        PM                          ,  --           158
        APC                         ,  --           159
  
        -- Other graphic characters:
  
        No_Break_Space              ,  --           160
        Inverted_Exclamation        ,  --           161
        Cent_Sign                   ,  --           162
        Pound_Sign                  ,  --           163
        Currency_Sign               ,  --           164
        Yen_Sign                    ,  --           165
        Broken_Bar                  ,  --           166
        Section_Sign                ,  --           167
        Diaeresis                   ,  --           168
        Copyright_Sign              ,  --           169
        Feminine_Ordinal_Indicator  ,  --           170
        Left_Angle_Quotation        ,  --           171
        Not_Sign                    ,  --           172
        Soft_Hyphen                 ,  --           173
        Registered_Trade_Mark_Sign  ,  --           174
        Macron                      ,  --           175
  
        Degree_Sign                 ,  --           176
        Plus_Minus_Sign             ,  --           177
        Superscript_Two             ,  --           178
        Superscript_Three           ,  --           179
        Acute                       ,  --           180
        Micro_Sign                  ,  --           181
        Pilcrow_Sign                ,  --           182
        Middle_Dot                  ,  --           183
        Cedilla                     ,  --           184
        Superscript_One             ,  --           185
        Masculine_Ordinal_Indicator ,  --           186
        Right_Angle_Quotation       ,  --           187
        Fraction_One_Quarter        ,  --           188
        Fraction_One_Half           ,  --           189
        Fraction_Three_Quarters     ,  --           190
        Inverted_Question           ,  --           191

        'A'                         ,  --           192
        'A'                         ,  --           193
        'A'                         ,  --           194
        'A'                         ,  --           195
        'A'                         ,  --           196
        'A'                         ,  --           197
        UC_AE_Diphthong             ,  --           198
        'C'                         ,  --           199
        'E'                         ,  --           200
        'E'                         ,  --           201
        'E'                         ,  --           202
        'E'                         ,  --           203
        'I'                         ,  --           204
        'I'                         ,  --           205
        'I'                         ,  --           206
        'I'                         ,  --           207

        UC_Icelandic_Eth            ,  --           208
        'N'                         ,  --           209
        'O'                         ,  --           210
        'O'                         ,  --           211
        'O'                         ,  --           212
        'O'                         ,  --           213
        'O'                         ,  --           214
        Multiplication_Sign         ,  --           215
        'O'                         ,  --           216
        'U'                         ,  --           217
        'U'                         ,  --           218
        'U'                         ,  --           219
        'U'                         ,  --           220
        'Y'                         ,  --           221
        UC_Icelandic_Thorn          ,  --           222
        LC_German_Sharp_S           ,  --           223
        
	LC_A                        ,  -- 'a'       224
        LC_A                        ,  -- 'a'       225
        LC_A                        ,  -- 'a'       226
        LC_A                        ,  -- 'a'       227
        LC_A                        ,  -- 'a'       228
        LC_A                        ,  -- 'a'       229
        LC_AE_Diphthong             ,  --           230
        LC_C                        ,  -- 'c'       231
        LC_E                        ,  -- 'e'       232
        LC_E                        ,  -- 'e'       233
        LC_E                        ,  -- 'e'       234
        LC_E                        ,  -- 'e'       235
        LC_I                        ,  -- 'i'       236
        LC_I                        ,  -- 'i'       237
        LC_I                        ,  -- 'i'       238
        LC_I                        ,  --           239
        
	LC_Icelandic_Eth            ,  --           240
        LC_N                        ,  -- 'n'       241
        LC_O                        ,  -- 'o'       242
        LC_O                        ,  -- 'o'       243
        LC_O                        ,  -- 'o'       244
        LC_O                        ,  -- 'o'       245
        LC_O                        ,  -- 'o'       246
        Division_Sign               ,  --           247
        LC_O                        ,  -- 'o'       248
        LC_U                        ,  -- 'u'       249
        LC_U                        ,  -- 'u'       250
        LC_U                        ,  -- 'u'       251
        LC_U                        ,  -- 'u'       252
        LC_Y                        ,  -- 'y'       253
        LC_Icelandic_Thorn          ,  --           254
        LC_Y                        ); -- 'y'       255
 
end Ada.Strings.Maps.Constants;
