-- $Source: /user/cp_source/ObjectAda/rts/common/strings_characters/ada.strings.unbounded.spc,v $
-- $Revision: 1.7 $ $Date: 1999/11/16 17:45:53 $ $Author: michael $
--        Copyright (C) 1995 by Intermetrics, Inc.

with Ada.Strings.Maps;
with Ada.Finalization;
with Ada.Streams;
with System;

package Ada.Strings.Unbounded is
    pragma Suppress( Elaboration_Check );
    pragma Preelaborate (Unbounded);

    type Unbounded_String is private;

    Null_Unbounded_String : constant Unbounded_String;

    function Length (Source : Unbounded_String) return Natural;

    type String_Access is access all String;
    procedure Free (X : in out String_Access);

 -- Conversion, Concatenation, and Selection functions

    function To_Unbounded_String (Source : String)
       return Unbounded_String;
    function To_Unbounded_String (Length : in Natural)
       return Unbounded_String;

    function To_String (Source : Unbounded_String) return String;

    procedure Append (Source   : in out Unbounded_String;
                      New_Item : in Unbounded_String);

    procedure Append (Source   : in out Unbounded_String;
                      New_Item : in String);

    procedure Append (Source   : in out Unbounded_String;
                      New_Item : in Character);

    function "&" (Left, Right : Unbounded_String) return Unbounded_String;

    function "&" (Left  : in Unbounded_String; Right : in String)
       return  Unbounded_String;

    function "&" (Left  : in String; Right : in Unbounded_String)
       return  Unbounded_String;

    function "&" (Left  : in Unbounded_String; Right : in Character)
       return  Unbounded_String;

    function "&" (Left  : in Character; Right : in Unbounded_String)
       return  Unbounded_String;

    function Element (Source : in Unbounded_String;
                      Index  : in Positive)
       return   Character;

    procedure Replace_Element (Source : in out Unbounded_String;
                               Index  : in Positive;
                               By     : Character);

    function Slice (Source : in Unbounded_String;
                    Low    : in Positive;
                    High   : in Natural)
       return   String;

    function "=" (Left, Right : in Unbounded_String) return Boolean;

    function "=" (Left  : in Unbounded_String; Right : in String)
       return  Boolean;

    function "=" (Left  : in String; Right : in Unbounded_String)
       return  Boolean;

    function "<" (Left, Right : in Unbounded_String) return Boolean;

    function "<" (Left  : in Unbounded_String; Right : in String)
       return  Boolean;

    function "<" (Left  : in String; Right : in Unbounded_String)
       return  Boolean;

    function "<=" (Left, Right : in Unbounded_String) return Boolean;

    function "<=" (Left  : in Unbounded_String; Right : in String)
       return  Boolean;

    function "<=" (Left  : in String; Right : in Unbounded_String)
       return  Boolean;

    function ">" (Left, Right : in Unbounded_String) return Boolean;

    function ">" (Left  : in Unbounded_String; Right : in String)
       return  Boolean;

    function ">" (Left  : in String; Right : in Unbounded_String)
       return  Boolean;

    function ">=" (Left, Right : in Unbounded_String) return Boolean;

    function ">=" (Left  : in Unbounded_String; Right : in String)
       return  Boolean;

    function ">=" (Left  : in String; Right : in Unbounded_String)
       return  Boolean;

 -- Search subprograms

    function Index (Source   : in Unbounded_String;
                    Pattern  : in String;
                    Going    : in Direction := Forward;
                    Mapping  : in Maps.Character_Mapping
         := Maps.Identity)
       return     Natural;

    function Index (Source   : in Unbounded_String;
                    Pattern  : in String;
                    Going    : in Direction := Forward;
                    Mapping  : in Maps.Character_Mapping_Function)
       return     Natural;

    function Index (Source : in Unbounded_String;
                    Set    : in Maps.Character_Set;
                    Test   : in Membership := Inside;
                    Going  : in Direction  := Forward)
       return   Natural;

    function Index_Non_Blank (Source : in Unbounded_String;
                              Going  : in Direction := Forward)
       return   Natural;

    function Count (Source  : in Unbounded_String;
                    Pattern : in String;
                    Mapping : in Maps.Character_Mapping
        := Maps.Identity)
       return    Natural;

    function Count (Source   : in Unbounded_String;
                    Pattern  : in String;
                    Mapping  : in Maps.Character_Mapping_Function)
       return     Natural;

    function Count (Source : in Unbounded_String;
                    Set    : in Maps.Character_Set)
       return   Natural;

    procedure Find_Token (Source : in Unbounded_String;
                          Set    : in Maps.Character_Set;
                          Test   : in Membership;
                          First  : out Positive;
                          Last   : out Natural);

 -- String translation subprograms

    function Translate (Source  : in Unbounded_String;
                        Mapping : in Maps.Character_Mapping)
       return    Unbounded_String;

    procedure Translate (Source  : in out Unbounded_String;
                         Mapping : Maps.Character_Mapping);

    function Translate (Source  : in Unbounded_String;
                        Mapping : in Maps.Character_Mapping_Function)
       return    Unbounded_String;

    procedure Translate (Source  : in out Unbounded_String;
                         Mapping : in Maps.Character_Mapping_Function);

 -- String transformation subprograms

    function Replace_Slice (Source : in Unbounded_String;
                            Low    : in Positive;
                            High   : in Natural;
                            By     : in String)
       return   Unbounded_String;

    procedure Replace_Slice (Source   : in out Unbounded_String;
                             Low      : in Positive;
                             High     : in Natural;
                             By       : in String);

    function Insert (Source   : in Unbounded_String;
                     Before   : in Positive;
                     New_Item : in String)
       return     Unbounded_String;

    procedure Insert (Source   : in out Unbounded_String;
                      Before   : in Positive;
                      New_Item : in String);

    function Overwrite (Source   : in Unbounded_String;
                        Position : in Positive;
                        New_Item : in String)
       return     Unbounded_String;

    procedure Overwrite (Source    : in out Unbounded_String;
                         Position  : in Positive;
                         New_Item  : in String);

    function Delete (Source  : in Unbounded_String;
                     From    : in Positive;
                     Through : in Natural)
       return    Unbounded_String;

    procedure Delete (Source  : in out Unbounded_String;
                      From    : in Positive;
                      Through : in Natural);

    function Trim (Source : in Unbounded_String;
                   Side   : in Trim_End)
       return   Unbounded_String;

    procedure Trim (Source : in out Unbounded_String;
                    Side   : in Trim_End);

    function Trim (Source : in Unbounded_String;
                   Left   : in Maps.Character_Set;
                   Right  : in Maps.Character_Set)
       return   Unbounded_String;

    procedure Trim (Source : in out Unbounded_String;
                    Left   : in Maps.Character_Set;
                    Right  : in Maps.Character_Set);

    function Head (Source : in Unbounded_String;
                   Count  : in Natural;
                   Pad    : in Character := Space)
       return   Unbounded_String;

    procedure Head (Source : in out Unbounded_String;
                    Count  : in Natural;
                    Pad    : in Character := Space);

    function Tail (Source : in Unbounded_String;
                   Count  : in Natural;
                   Pad    : in Character := Space)
       return   Unbounded_String;

    procedure Tail (Source : in out Unbounded_String;
                    Count  : in Natural;
                    Pad    : in Character := Space);

    function "*" (Left  : in Natural;
                  Right : in Character)
       return  Unbounded_String;

    function "*" (Left  : in Natural;
                  Right : in String)
       return  Unbounded_String;

    function "*" (Left  : in Natural;
                  Right : in Unbounded_String)
       return  Unbounded_String;

private

    -- We make only one variant of Unbounded_String controlled,
    -- so we can create a preelaborable Null_Unbounded_String constant.

    -- Use a discriminated record in the heap rather than an unconstrained
    -- string to save space and some checking overhead
    type String_Buffer(Buffer_Len : Natural) is record
        Buffer : String(1..Buffer_len);
    end record;
    type String_Buffer_Ptr is access String_Buffer;

    -- Wrap the pointer in a controlled type
    type Controlled_String_Ptr is new Ada.Finalization.Controlled with record
        Pointer : String_Buffer_Ptr;
    end record;

    -- Provide adjust/finalize so assignment works right
    procedure Adjust     (Object : in out Controlled_String_Ptr);
    procedure Finalize   (Object : in out Controlled_String_Ptr);

    -- Provide "=" so it composes properly
    -- NOTE: this is not really required by RM, but users may
    --   expect to be able to use the "implicit" composability of "="
    --   and not have the predefined equality reemerge in generics.
    --   Also, AI95-00123 requires that all language-defined types
    --   compose properly.
    function "="(Left, Right : Controlled_String_Ptr) return Boolean;

    Max_Short : constant := 2 * Integer'Size / Character'Size;
       -- two words of characters, typically 8.

    Zeroes : constant String(1..Max_Short) := (others => Character'Val(0));

  type Unbounded_String_Contents(Is_Long : Boolean := False) is
    record
      Length : Natural := 0;
      case Is_Long is
        when false =>
          Short : String(1..Max_Short) := Zeroes;
        when true =>
          Long : Controlled_String_Ptr;
      end case;
    end record;
  pragma Pack(Unbounded_String_Contents);

  type Unbounded_String is
    record
      Contents: Unbounded_String_Contents;
  end record;
    pragma Pack(Unbounded_String);  -- So Is_Long and Length in same word

    -- Add special stream I/O routines
    procedure Stream_Read(
      Stream : access Ada.Streams.Root_Stream_Type'Class;
      Item : out Unbounded_String);

    procedure Stream_Write(
      Stream : access Ada.Streams.Root_Stream_Type'Class;
      Item : in Unbounded_String);

    for Unbounded_String'Read use Stream_Read;
    for Unbounded_String'Write use Stream_Write;

    Null_Unbounded_String : constant Unbounded_String :=
        --Unbounded_String'(Contents => Null_Unbounded_String_Contents);
        Unbounded_String'(Contents =>
                           Unbounded_String_Contents'(Is_Long=>False, Length=>0,
                                                         Short => (others => Character'Val(0))));

end Ada.Strings.Unbounded;
