-- $Source: /user/cp_source/ObjectAda/rts/common/strings_characters/ada.strings.wide_unbounded.spc,v $ 
-- $Revision: 1.5 $ $Date: 2005/04/12 14:19:54 $ $Author: shancher $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

with Ada.Strings.Wide_Maps;
with Ada.Finalization;
with System;

package Ada.Strings.Wide_Unbounded is
    pragma Suppress( Elaboration_Check );
    pragma Preelaborate (Wide_Unbounded);

    type Unbounded_Wide_String is private;

    Null_Unbounded_Wide_String : constant Unbounded_Wide_String;

    function Length (Source : Unbounded_Wide_String) return Natural;

    type Wide_String_Access is access all Wide_String;
    procedure Free (X : in out Wide_String_Access);
   
 -- Conversion, Concatenation, and Selection functions

    function To_Unbounded_Wide_String (Source : Wide_String)     
       return Unbounded_Wide_String;
    function To_Unbounded_Wide_String (Length : in Natural) 
       return Unbounded_Wide_String;

    function To_Wide_String (Source : Unbounded_Wide_String) return Wide_String;

    procedure Append (Source   : in out Unbounded_Wide_String;
                      New_Item : in Unbounded_Wide_String);

    procedure Append (Source   : in out Unbounded_Wide_String;
                      New_Item : in Wide_String);

    procedure Append (Source   : in out Unbounded_Wide_String;
                      New_Item : in Wide_Character);

    function "&" (Left, Right : Unbounded_Wide_String) return Unbounded_Wide_String;

    function "&" (Left  : in Unbounded_Wide_String; Right : in Wide_String)
       return  Unbounded_Wide_String;

    function "&" (Left  : in Wide_String; Right : in Unbounded_Wide_String)
       return  Unbounded_Wide_String;

    function "&" (Left  : in Unbounded_Wide_String; Right : in Wide_Character)
       return  Unbounded_Wide_String;

    function "&" (Left  : in Wide_Character; Right : in Unbounded_Wide_String)
       return  Unbounded_Wide_String;

    function Element (Source : in Unbounded_Wide_String;
                      Index  : in Positive)
       return   Wide_Character;

    procedure Replace_Element (Source : in out Unbounded_Wide_String;
                               Index  : in Positive;
                               By     : Wide_Character);

    function Slice (Source : in Unbounded_Wide_String;
                    Low    : in Positive;
                    High   : in Natural)
       return   Wide_String;

    function "=" (Left, Right : in Unbounded_Wide_String) return Boolean;

    function "=" (Left  : in Unbounded_Wide_String; Right : in Wide_String)
       return  Boolean;

    function "=" (Left  : in Wide_String; Right : in Unbounded_Wide_String)
       return  Boolean;

    function "<" (Left, Right : in Unbounded_Wide_String) return Boolean;

    function "<" (Left  : in Unbounded_Wide_String; Right : in Wide_String)
       return  Boolean;

    function "<" (Left  : in Wide_String; Right : in Unbounded_Wide_String)
       return  Boolean;

    function "<=" (Left, Right : in Unbounded_Wide_String) return Boolean;

    function "<=" (Left  : in Unbounded_Wide_String; Right : in Wide_String)
       return  Boolean;

    function "<=" (Left  : in Wide_String; Right : in Unbounded_Wide_String)
       return  Boolean;

    function ">" (Left, Right : in Unbounded_Wide_String) return Boolean;

    function ">" (Left  : in Unbounded_Wide_String; Right : in Wide_String)
       return  Boolean;

    function ">" (Left  : in Wide_String; Right : in Unbounded_Wide_String)
       return  Boolean;

    function ">=" (Left, Right : in Unbounded_Wide_String) return Boolean;

    function ">=" (Left  : in Unbounded_Wide_String; Right : in Wide_String)
       return  Boolean;

    function ">=" (Left  : in Wide_String; Right : in Unbounded_Wide_String)
       return  Boolean;

 -- Search subprograms

    function Index (Source   : in Unbounded_Wide_String;
                    Pattern  : in Wide_String;
                    Going    : in Direction := Forward;
                    Mapping  : in Wide_Maps.Wide_Character_Mapping 
				 := Wide_Maps.Identity)
       return     Natural;

    function Index (Source   : in Unbounded_Wide_String;
                    Pattern  : in Wide_String;
                    Going    : in Direction := Forward;
                    Mapping  : in Wide_Maps.Wide_Character_Mapping_Function)
       return     Natural;

    function Index (Source : in Unbounded_Wide_String;
                    Set    : in Wide_Maps.Wide_Character_Set;
                    Test   : in Membership := Inside;
                    Going  : in Direction  := Forward)
       return   Natural;

    function Index_Non_Blank (Source : in Unbounded_Wide_String;
                              Going  : in Direction := Forward)
       return   Natural;

    function Count (Source  : in Unbounded_Wide_String;
                    Pattern : in Wide_String;
                    Mapping : in Wide_Maps.Wide_Character_Mapping 
				:= Wide_Maps.Identity)
       return    Natural;

    function Count (Source   : in Unbounded_Wide_String;
                    Pattern  : in Wide_String;
                    Mapping  : in Wide_Maps.Wide_Character_Mapping_Function)
       return     Natural;

    function Count (Source : in Unbounded_Wide_String;
                    Set    : in Wide_Maps.Wide_Character_Set)
       return   Natural;

    procedure Find_Token (Source : in Unbounded_Wide_String;
                          Set    : in Wide_Maps.Wide_Character_Set;
                          Test   : in Membership;
                          First  : out Positive;
                          Last   : out Natural);

 -- Wide_String translation subprograms

    function Translate (Source  : in Unbounded_Wide_String;
                        Mapping : in Wide_Maps.Wide_Character_Mapping)
       return    Unbounded_Wide_String;

    procedure Translate (Source  : in out Unbounded_Wide_String;
                         Mapping : Wide_Maps.Wide_Character_Mapping);

    function Translate (Source  : in Unbounded_Wide_String;
                        Mapping : in Wide_Maps.Wide_Character_Mapping_Function)
       return    Unbounded_Wide_String;

    procedure Translate (Source  : in out Unbounded_Wide_String;
                         Mapping : in Wide_Maps.Wide_Character_Mapping_Function);

 -- Wide_String transformation subprograms

    function Replace_Slice (Source : in Unbounded_Wide_String;
                            Low    : in Positive;
                            High   : in Natural;
                            By     : in Wide_String)
       return   Unbounded_Wide_String;

    procedure Replace_Slice (Source   : in out Unbounded_Wide_String;
                             Low      : in Positive;
                             High     : in Natural;
                             By       : in Wide_String);

    function Insert (Source   : in Unbounded_Wide_String;
                     Before   : in Positive;
                     New_Item : in Wide_String)
       return     Unbounded_Wide_String;

    procedure Insert (Source   : in out Unbounded_Wide_String;
                      Before   : in Positive;
                      New_Item : in Wide_String);

    function Overwrite (Source   : in Unbounded_Wide_String;
                        Position : in Positive;
                        New_Item : in Wide_String)
       return     Unbounded_Wide_String;

    procedure Overwrite (Source    : in out Unbounded_Wide_String;
                         Position  : in Positive;
                         New_Item  : in Wide_String);

    function Delete (Source  : in Unbounded_Wide_String;
                     From    : in Positive;
                     Through : in Natural)
       return    Unbounded_Wide_String;

    procedure Delete (Source  : in out Unbounded_Wide_String;
                      From    : in Positive;
                      Through : in Natural);

    function Trim (Source : in Unbounded_Wide_String;
                   Side   : in Trim_End)
       return   Unbounded_Wide_String;

    procedure Trim (Source : in out Unbounded_Wide_String;
                    Side   : in Trim_End);

    function Trim (Source : in Unbounded_Wide_String;
                   Left   : in Wide_Maps.Wide_Character_Set;
                   Right  : in Wide_Maps.Wide_Character_Set)
       return   Unbounded_Wide_String;

    procedure Trim (Source : in out Unbounded_Wide_String;
                    Left   : in Wide_Maps.Wide_Character_Set;
                    Right  : in Wide_Maps.Wide_Character_Set);

    function Head (Source : in Unbounded_Wide_String;
                   Count  : in Natural;
                   Pad    : in Wide_Character := Wide_Space)
       return   Unbounded_Wide_String;

    procedure Head (Source : in out Unbounded_Wide_String;
                    Count  : in Natural;
                    Pad    : in Wide_Character := Wide_Space);

    function Tail (Source : in Unbounded_Wide_String;
                   Count  : in Natural;
                   Pad    : in Wide_Character := Wide_Space)
       return   Unbounded_Wide_String;

    procedure Tail (Source : in out Unbounded_Wide_String;
                    Count  : in Natural;
                    Pad    : in Wide_Character := Wide_Space);

    function "*" (Left  : in Natural;
                  Right : in Wide_Character)
       return  Unbounded_Wide_String;

    function "*" (Left  : in Natural;
                  Right : in Wide_String)
       return  Unbounded_Wide_String;

    function "*" (Left  : in Natural;
                  Right : in Unbounded_Wide_String)
       return  Unbounded_Wide_String;

private

    -- We make only one variant of Unbounded_Wide_String controlled,
    -- so we can create a preelaborable Null_Unbounded_Wide_String constant.

    -- Use a discriminated record in the heap rather than an unconstrained 
    -- string to save space and some checking overhead
    type Wide_String_Buffer(Buffer_Len : Natural) is record
	Buffer : Wide_String(1..Buffer_len);
    end record;
    type Wide_String_Buffer_Ptr is access Wide_String_Buffer;

    -- Wrap the pointer in a controlled type
    type Controlled_Wide_String_Ptr is 
      new Ada.Finalization.Controlled with record
        Pointer : Wide_String_Buffer_Ptr;
    end record;

    -- Provide adjust/finalize so assignment works right
    procedure Adjust     (Object : in out Controlled_Wide_String_Ptr);
    procedure Finalize   (Object : in out Controlled_Wide_String_Ptr);
        
    Max_Short : constant := 2 * (Integer'Size / Wide_Character'Size);
       -- two words of wide characters, typically 4.
 
    type Unbounded_Wide_String(Is_Long : Boolean := False) is record
        Length : Natural := 0;
        case Is_Long is
            when false =>
                Short : Wide_String(1..Max_Short) := (others => 
                    Wide_Character'Val(0));
            when true =>
                Long : Controlled_Wide_String_Ptr;
        end case;
    end record;
    pragma Pack(Unbounded_Wide_String);  -- So Is_Long and Length in same word
 
    Null_Unbounded_Wide_String : constant Unbounded_Wide_String := 
        (Is_Long => False,
         Length  => 0,
         Short   => (others => Wide_Character'Val(0)));
 
end Ada.Strings.Wide_Unbounded;
