-- $Source: /user/cp_source/ObjectAda/rts/common/systems_programming/ada.tags.spc,v $
-- $Revision: 1.3 $ $Date: 1998/12/16 19:38:04 $ $Author: curt $
--        Copyright (C) 1995 by Intermetrics, Inc.
-- $Source: /user/cp_source/ObjectAda/rts/common/systems_programming/ada.tags.spc,v $ 
-- $Revision: 1.3 $ $Date: 1998/12/16 19:38:04 $ $Author: curt $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

pragma ada_child;
package Ada.Tags is
    pragma Suppress( Elaboration_Check );

    type Tag is private;
    
    function Expanded_Name(T : Tag) return String;
    pragma Export(Ada, Expanded_Name, "expanded_name");
    pragma Global(Expanded_Name);
    
    function External_Tag(T : Tag) return String;
    pragma Export(Ada, External_Tag, "external_tag");
    pragma Global(External_Tag);

    function Internal_Tag(External : String) return Tag;
    pragma Export(Ada, Internal_Tag, "internal_tag");
    pragma Global(Internal_Tag);
    
    Tag_Error : exception;
    
private

    type Tag_Type_Descriptor;
    type Tag is access Tag_Type_Descriptor;

    -- The following types and routines are used by the 
    -- compiler-generated code only:

    type Tag_Type_Link;
    type Tag_Type_Link_Access is access all Tag_Type_Link;

    procedure RTS_Insert_Tag_Descriptor(Tag_Link : in Tag_Type_Link_access);
    pragma Export(Ada,
    	          RTS_Insert_Tag_Descriptor,
		  External_Name => "rts_insert_tag_descriptor");
    pragma Global(RTS_Insert_Tag_Descriptor);
    -- Insert the given tag into the hash table.  This function is called
    -- when the tagged type is elaborated.

    --
    -- Classwide Finalization
    --
    -- The following routines provide support for classwide finalization.
    -- They are here because they depend substantially on the format of
    -- the tag descriptor, but depend very little on any details of the
    -- finalization mechanisms.

    type Normal_Tagged_Obj is record
	Tag_Descpt : Tag;
    end record;

    procedure RTS_Adjust_Classwide_Object(Obj : in out Normal_Tagged_Obj);
    pragma Global(RTS_Adjust_Classwide_Object);
    pragma Export(Ada, RTS_Adjust_Classwide_Object, 
      External_Name => "rts_adjust_classwide_object");
    -- Overview:
    --   This operation dispatches to the Adjust_Whole_Object
    --   operation, based on the tag of the object.
    --   This presumes the tag is in the first word of the object.
    --   The compiler must generate a special routine if the
    --   tag is not the first word of the object.
    
    procedure RTS_Finalize_Classwide_Object(Obj : in out Normal_Tagged_Obj);
    pragma Global(RTS_Finalize_Classwide_Object);
    pragma Export(Ada, RTS_Finalize_Classwide_Object, 
      External_Name => "rts_finalize_classwide_object");
    -- Overview:
    --   This operation dispatches to the Finalize_Whole_Object
    --   operation, based on the tag of the object.
    --   This presumes the tag is in the first word of the object.
    --   The compiler must generate a special routine if the
    --   tag is not the first word of the object.


    procedure RTS_Null_Operation(Obj : in out Normal_Tagged_Obj);
    pragma Global(RTS_Null_Operation);
    pragma Export(Ada, RTS_Null_Operation, 
      External_Name => "rts_null_operation");
    -- Overview:
    --   This operation just returns.  It is put into
    --   Adjust_Whole_Object and Finalize_Whole_Object slots
    --   for tagged types that have no controlled parts.
    -- NOTE: This used to be in the finalization package, but
    --   that brings in too much code.

end Ada.Tags;
