-- $Source: /user/cp_source/ObjectAda/rts/common/exceptions/system.rts.exception_types.dat,v $
-- $Revision: 1.3 $ $Date: 2000/04/11 16:38:15 $ $Author: curt $
--        Copyright (C) 1995 by Intermetrics, Inc.

with System.Storage_Elements;
with Ada.Unchecked_Conversion;
with System.RTS.TGT.Exception_Reason_Pkg;

package System.RTS.Exception_Types is

    pragma Suppress(Elaboration_Check);
    pragma Preelaborate(System.RTS.Exception_Types);

    package ERP renames System.RTS.TGT.Exception_Reason_Pkg;

    type Exception_Name_String is array( Natural range <> ) of Character;
    pragma pack( Exception_Name_String );

    type Exception_Name_Type( Len : Natural := 0 ) is
        record
            Val          : Exception_Name_String( 1 .. Len );
        end record;
    for Exception_Name_Type use
        record
            Len at 0 range 0 .. Word_Size - 1;
        end record;


    type Exception_Id is access Exception_Name_Type;

    Null_Id : constant Exception_Id := null;

    type Frame is new System.Storage_Elements.Integer_Address;

    Null_Frame : constant Frame := 0;
          -- The above should match the value of Null Address.

    type Frame_Array is array(Positive range <>) of Frame;

    Stack_Trace_Num_Frames : constant := 8;

    type Stack_Trace is
        record
            Frames_Recorded : Integer := 0;
            Frames          : Frame_Array(1 .. Stack_Trace_Num_Frames) :=
                                ( others => Null_Frame );
            Handler_Frame   : Frame := Null_Frame;
        end record;

    Null_Stack_Trace : constant Stack_Trace :=
        ( Frames_Recorded => 0,
          Frames          => ( others => Null_Frame ),
          Handler_Frame   => Null_Frame );

    subtype Message_Buf_Size_Range is Integer range -1 .. Integer'Last;
    No_Message_Buffer : constant Message_Buf_Size_Range := -1;

    type Minimal_Exception_Occurrence is
        record
            Identity          : Exception_Id := Null_Id;
            Stack_Info        : Stack_Trace := Null_Stack_Trace;
            Free_When_Handled : Boolean := False;
            Reason            : ERP.Reason_Code := ERP.No_Reason;
            Message_Buf_Size  : Message_Buf_Size_Range := -1;
            Message_Length    : Message_Buf_Size_Range := -1;
        end record;

    generic
        Buffer_Size : Message_Buf_Size_Range;
    package Generic_Occurrence_Minimal is
        type Exception_Occurrence is
            record
                Info    : Minimal_Exception_Occurrence := (
                              Null_Id,
                              Null_Stack_Trace,
                              False,
                              ERP.No_Reason,
                              Buffer_Size,
                              0 );
                Message : String( 1 .. Buffer_Size );
            end record;
        type Exception_Occurrence_Access is access all Exception_Occurrence;
    end Generic_Occurrence_Minimal;

    package RTS_EO is new Generic_Occurrence_Minimal(0);

    subtype Exception_Occurrence is RTS_EO.Exception_Occurrence;
    subtype Exception_Occurrence_Access is RTS_EO.Exception_Occurrence_Access;
    generic
        Buffer_Size : Message_Buf_Size_Range;
    package Generic_Occurrence is
        type Exception_Occurrence is
            record
                Info    : Minimal_Exception_Occurrence := (
                              Null_Id,
                              Null_Stack_Trace,
                              False,
                              ERP.No_Reason,
                              Buffer_Size,
                              0 );
                Message : String( 1 .. Buffer_Size );
            end record;
        type Exception_Occurrence_Access is access all Exception_Occurrence;

        function "+" is new Ada.Unchecked_Conversion(
            Exception_Occurrence_Access,
            System.RTS.Exception_Types.Exception_Occurrence_Access);
        function "+" is new Ada.Unchecked_Conversion(
            System.RTS.Exception_Types.Exception_Occurrence_Access,
            Exception_Occurrence_Access);
        function "+" is new Ada.Unchecked_Conversion(
            Exception_Occurrence,
            System.RTS.Exception_Types.Exception_Occurrence);
        function "+" is new Ada.Unchecked_Conversion(
            System.RTS.Exception_Types.Exception_Occurrence,
            Exception_Occurrence);
    end Generic_Occurrence;

end System.RTS.Exception_Types;
