-- $Source: /user/cp_source/ObjectAda/rts/common/tasking/system.rts.interface_types.dat,v $ 
-- $Revision: 1.1 $ $Date: 1996/02/26 20:51:03 $ $Author: peignot $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

package System.RTS.Interface_Types is

    pragma Suppress( Elaboration_Check );

--* OVERVIEW
-- This package defines types of parameters which occur in operations in the
-- generated code/RTS interface. 

    subtype Code_Address is Address;
    subtype Data_Address is Address;

    subtype Param_Block_Ptr is Data_Address;
    subtype Pr_Subp_Ptr is Code_Address;

    type Time_Type_Enum is (
      From_Calendar,		-- Ada.Calendar.Time
      From_Real_Time,		-- Ada.Real_Time.Time
      From_Timer_Record);	-- TGT.Kernel.Timers.Timer_Record'Class
      -- designates the kind of the time type in Absolute_Delay_Handle.all

    subtype Caller_Count is Natural;

    subtype Simple_Handle is Data_Address;
      -- pointer to a data structure that is unknown outside of the RTS.
      -- e.g., masters, TCBS.  The body of RTS_Tasking_Interface takes
      -- care of conversion between this type and a type used internally
      -- by the RTS (typically access to a record type).

    subtype Activation_Handle is Simple_Handle;
    subtype ECD_Handle is Simple_Handle;
    subtype Master_Handle is Simple_Handle;
    subtype PO_Handle is Simple_Handle;
    subtype Pr_Type_Desc_Handle is Simple_Handle;
    subtype Task_Obj_Handle is Simple_Handle;
    subtype Handler_Desc_Handle is Simple_Handle;
    subtype Handler_Info_Handle is Simple_Handle;
    subtype Select_Handle is Simple_Handle;
    subtype Setjmp_Buffer_Handle is Simple_Handle;
    subtype Absolute_Delay_Handle is Simple_Handle;
      -- A pointer to different types of absolute time.
      -- The kind of time type is conveyed in a separate Time_Type_Enum
      -- parameter for predefined time types (calendar.time, real_time.time).
      -- User-defined time types are wrapped by the compiler into
      -- a tagged type derived from System.RTS.TGT.Kernel.Timers.Timer_Record.

    subtype Task_Entry_Index is Natural;
    subtype Task_Entry_Point is Code_Address;

    subtype Entry_Name_Index is Natural;
    subtype Entry_Member_Index is Integer;

    subtype Accept_Alt_Index is Natural;

    subtype Priority_Type is Integer;
    Priority_Not_Specified : constant Priority_Type := Priority_Type'First;

    subtype Interrupt_Id is Integer;

end System.RTS.Interface_Types;
