-- $Source: /user/cp_source/ObjectAda/rts/common/tasking/system.rts.protected_type_descriptor_pkg.dat,v $
-- $Revision: 1.1 $ $Date: 1998/04/08 19:09:16 $ $Author: ada $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

package System.RTS.Protected_Type_Descriptor_Pkg is

    pragma Suppress( Elaboration_Check );

--* OVERVIEW
-- This package contains representation types for protected types
-- and objects required by the Patriot Ada RTM. For each protected
-- type declared by the user, an object of type Pr_Type_Desc is
-- allocated and initialized by the compiler. For each protected
-- object declared (or allocated), an object of type PO_Rec is
-- allocated by the compiler as part of the protected object, but
-- initialized by the RTS.  The Pr_Type_Desc structure is shared
-- by all protected objects of the same type.

    --============ SIMPLE TYPES ============

    subtype Code_Address is Address;

    type Entry_Name_Index is new Natural;

    subtype Entry_Index is
	  Entry_Name_Index range 0 .. Word_Size - 1;

    type PT_Kind_Enum is
      ( PT_No_Entries,
      PT_Small,
      PT_Large );

    type Handler_Index is new Natural;

    --============ SIMPLE ARRAY TYPES ============

    type Code_Array is array( Entry_Name_Index range <> ) of Code_Address;
    type Handler_Array is array( Handler_Index range <> ) of Code_Address;

    --============ RECORD TYPE DECLARATIONS ============

    type Pr_Type_Desc( PT_Kind        : PT_Kind_Enum;
                       Max_Name_Index : Entry_Name_Index );

    type Handler_List_Type( Num_Handlers : Handler_Index );
    type Handler_List_Access is access all Handler_List_Type;

    --============ RECORD TYPES ============

    -- The index of a handler has no use except for the construction
    -- of a simple list, unlike the case of an entry name index.
    type Handler_List_Type( Num_Handlers : Handler_Index ) is
    record
        Handler : Handler_Array( 1 .. Num_Handlers );
    End record;
 
    type Pr_Type_Desc( PT_Kind        : PT_Kind_Enum;
                       Max_Name_Index : Entry_Name_Index ) is
    record
        Handler_List : Handler_List_Access;
        case PT_Kind is
            when PT_No_Entries       =>
                null;
            when PT_Small | PT_Large =>
                Depends_On_Count : Boolean;
                  -- some entry barrier depends on 'COUNT
                Entry_Code       : Code_Array(
		                  Entry_Name_Index'First .. Max_Name_Index );
                case PT_Kind is
                    when PT_Small =>
                        Compute_All_Barriers : Code_Address;
                    when PT_Large =>
                        Compute_Barriers     : Code_Array(
				  Entry_Name_Index'First .. Max_Name_Index );
                    when others   =>
                        null;
                end case;
        end case;
    end record;

end System.RTS.Protected_Type_Descriptor_Pkg;
