-- $Source: /user/cp_source/ObjectAda/rts/common/finalization/system.rts.tgt.frames.spc,v $ 
-- $Revision: 1.4 $ $Date: 2000/05/30 18:21:56 $ $Author: curt $ 
--        Copyright (C) 1995 by Intermetrics, Inc.   

with System.Storage_Elements;

package System.RTS.TGT.Frames is
    pragma Suppress( Elaboration_Check );
    pragma Preelaborate( System.RTS.TGT.Frames );
    -- The body of this package is dependent on the representation
    -- of stack frames.

    -- The body will presumably be provided by the implementor
    -- of the backend.

    type Frame_ID is private;
    -- Every stack frame is identified by a unique ID
    -- It will typically be the stack pointer or frame pointer.
    -- Every master record records the Frame_ID of its stack frame.

    Null_Frame_ID : constant Frame_ID;

    function Current_Frame return Frame_ID;
    -- This function returns the current frame ID.
    -- It is called from a routine that is called from
    -- generated code.
    -- (Perhaps should be Intrinsic; perhaps should be called
    -- directly from generated code).

    function Current_Return_Address return System.Address;
    -- Returns the return address of this function's caller,
    -- i.e. the address to which the caller will return when it
    -- finishes.
    -- This function should be called from an RTS routine which is
    -- called from generated code, and should return an address
    -- in the generated code.

private
    type Frame_ID is new System.Storage_Elements.Integer_Address;
    pragma Import (Intrinsic, Current_Frame, Link_Name => "*CURRENT_FRAME");
    pragma Import (Intrinsic, Current_Return_Address, "*CURRENT_RETURN_ADDRESS");

    Null_Frame_ID : constant Frame_ID := 0;
          -- Must match System.Null_Address;

end System.RTS.TGT.Frames;
