-- $Source: /user/cp_source/ObjectAda/rts/nt/system.spc,v $
-- $Revision: 1.1 $ $Date: 2000/10/20 23:57:54 $ $Author: curt $
-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
-- %                                                                       %
-- % "  This  unpublished work is protected both as a proprietary work and %
-- % under the Universal Copyright Convention  and the US Copyright Act of %
-- % 1976.  Its  distribution  and  access  are limited only to authorized %
-- % persons.  Copyright (c) 1996-2000, Aonix. All rights reserved.        %
-- %                                                                       %
-- % Unauthorized  use (including use to prepare other works), disclosure, %
-- % reproduction, or distribution may violate national criminal law. "    %
-- %                                                                       %
-- %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


package System is

    pragma Suppress( Elaboration_Check );
    pragma Pure (System);
    pragma Preelaborate (System);

    type Name is (S370,
                  I80X86,
                  I80386,
                  MC680X0,
                  Vax,
                  Transputer,
                  RS_6000,
                  MIPS,
                  HP9000_PA_RISC,
                  HP9000_300,
                  SPARC);
    System_Name  : constant Name := I80386;

    -- System-Dependent Named Numbers:

    Min_Int               : constant :=-2**31;
    Max_Int               : constant := 2**31 - 1;

    Max_Binary_Modulus    : constant := 2**32;
    Max_Nonbinary_Modulus : constant := 2**31 - 1;

    Max_Base_Digits       : constant := 15;
    Max_Digits            : constant := 15;

    Max_Mantissa          : constant := 31;
    Fine_Delta            : constant := 2.0 ** (-31);

    Tick                  : constant := 0.01;

    -- Storage-related Declarations:

    type Address is private;
    Null_Address          : constant Address;

    Storage_Unit          : constant := 8;
    Word_Size             : constant := 4 * Storage_Unit;
    Memory_Size           : constant := 2**32;

    -- Address Comparison:

    function "<"( Left : Address; Right  : Address ) return Boolean;
    function ">"( Left : Address; Right  : Address ) return Boolean;
    function "<="( Left : Address; Right : Address ) return Boolean;
    function ">="( Left : Address; Right : Address ) return Boolean;

    -- Other System-Dependent Declarations:

    type Bit_Order is ( High_Order_First, Low_Order_First );
    Default_Bit_Order : constant Bit_Order := Low_Order_First;

    -- Priority-related Declarations( see D.1 ):

    subtype ANY_PRIORITY is INTEGER range 1 .. 31;
    subtype PRIORITY is ANY_PRIORITY range
      ANY_PRIORITY'FIRST .. ANY_PRIORITY'LAST-1;
    subtype INTERRUPT_PRIORITY is ANY_PRIORITY
      range PRIORITY'LAST + 1 .. ANY_PRIORITY'LAST;

    Default_Priority : constant PRIORITY := (PRIORITY'FIRST + PRIORITY'LAST)/2;

    -- ---------------------------------------------------------------
    -- The following declarations are not part of the Ada95
    -- standard.  They are provide for backward compatibility
    -- with Ada83 products.

    function "-"( Left,Right : System.Address )
      return Integer;
    function "-"( Left : System.Address; Right : Integer )
      return System.Address;
    function "+"( Left : System.Address; Right : Integer )
      return System.Address;
    function "+"( Left : Integer; Right : System.Address )
      return System.Address;

    subtype Offset is Integer;
    subtype ADDRESS_STRING is STRING (1..8);

    function IMAGE ( LEFT : in ADDRESS ) return ADDRESS_STRING;
    function VALUE ( LEFT : in STRING ) return ADDRESS;

    type OBJECT_LENGTH is mod 2**32;

    -- These routines are provided to perform READ/WRITE operation
    -- in memory.
    -- Warning: These routines will give unexpected results if used with
    -- unconstrained types.
 
    generic
       type TARGET is private;
    function FETCH_FROM_ADDRESS (A : in ADDRESS) return TARGET;
 
    generic
       type TARGET is private;
    procedure ASSIGN_TO_ADDRESS (A : in ADDRESS; T : in TARGET);
 
    procedure MOVE (TO     : in ADDRESS;
                    FROM   : in ADDRESS;
                    LENGTH : in OBJECT_LENGTH);
 
    function "mod" (LEFT : in ADDRESS; RIGHT : in POSITIVE) return NATURAL;
 
    -- Returns the given address rounded to a specific value.
 
    type ROUND_DIRECTION is (DOWN, UP);
 
    function ROUND (VALUE     : in ADDRESS;
                    DIRECTION : in ROUND_DIRECTION;
                    MODULUS   : in POSITIVE) return ADDRESS;
 
    function AtoI(A : Address) return Integer;
    function ItoA(I : Integer) return Address;

private

   type Raw_Object is record
      null;
   end record;
   type Address is access Raw_Object;
   pragma Import( Intrinsic, Address, "*E_System_Address");
   Null_Address          : constant Address := null;

   pragma Import( Intrinsic, AtoI, "*E_Unchecked_Conversion");
   pragma Import( Intrinsic, ItoA, "*E_Unchecked_Conversion");
   pragma Import( Intrinsic, "<",  "*e_system.<" ); --
   pragma Import( Intrinsic, "<=", "*e_system.<=" ); --
   pragma Import( Intrinsic, ">",  "*e_system.>" ); --
   pragma Import( Intrinsic, ">=", "*e_system.>=" ); --
   pragma Import( Intrinsic, "+",  "*e_system_storage_elements.+" );
   pragma Import( Intrinsic, "-",  "*e_system_storage_elements.-" );
   pragma import ( Intrinsic, "mod", Link_Name => "*ObjectAda__address_mod");

end System;
